@@ -146,6 +146,34 @@ GET_ALL_PARAMS_PAYLOAD = (
     bytes([HAP_BLE_PROTOCOL_CONFIGURATION_REQUEST_TLV.GetAllParams]) + b"\x00"
 )
 
+# Higher priority means we will fetch these first
+CHAR_FETCH_PRIORITY = {
+    CharacteristicsTypes.MOTION_DETECTED: 100,
+    CharacteristicsTypes.CONTACT_STATE: 100,
+    CharacteristicsTypes.OCCUPANCY_DETECTED: 100,
+    CharacteristicsTypes.DOOR_STATE_CURRENT: 100,
+    CharacteristicsTypes.DOOR_STATE_TARGET: 100,
+    CharacteristicsTypes.INPUT_EVENT: 100,
+    CharacteristicsTypes.ON: 100,
+    CharacteristicsTypes.ACTIVE: 100,
+    CharacteristicsTypes.LOCK_MECHANISM_CURRENT_STATE: 100,
+    CharacteristicsTypes.LOCK_MECHANISM_TARGET_STATE: 100,
+    CharacteristicsTypes.LEAK_DETECTED: 90,
+    CharacteristicsTypes.SMOKE_DETECTED: 90,
+    CharacteristicsTypes.CARBON_MONOXIDE_DETECTED: 90,
+    CharacteristicsTypes.CARBON_DIOXIDE_DETECTED: 90,
+    CharacteristicsTypes.BRIGHTNESS: 90,
+    CharacteristicsTypes.HUE: 90,
+    CharacteristicsTypes.SATURATION: 90,
+    CharacteristicsTypes.COLOR_TEMPERATURE: 90,
+    CharacteristicsTypes.VERSION: -50,
+    CharacteristicsTypes.NAME: -50,
+    CharacteristicsTypes.THREAD_CONTROL_POINT: -100,
+    CharacteristicsTypes.THREAD_STATUS: -100,
+    CharacteristicsTypes.THREAD_NODE_CAPABILITIES: -100,
+    CharacteristicsTypes.THREAD_OPENTHREAD_VERSION: -100,
+}
+
 WrapFuncType = TypeVar("WrapFuncType", bound=Callable[..., Any])
 
 
@@ -643,8 +671,7 @@ class BlePairing(AbstractPairing):
             results = {(BLE_AID, iid): {"value": from_bytes(char, value)}}
             logger.debug("%s: Received notification: results = %s", self.name, results)
 
-            for listener in self.listeners:
-                listener(results)
+            self._callback_listeners(results)
             return
 
         logger.warning(
@@ -764,8 +791,7 @@ class BlePairing(AbstractPairing):
                 service_iid_bytes,
                 service_iid,
             )
-            s = accessory.add_service(normalize_uuid(service.uuid))
-            s.iid = service_iid
+            s = accessory.add_service(normalize_uuid(service.uuid), iid=service_iid)
 
             service_signature_char = service.get_characteristic(SERVICE_SIGNATURE_UUID)
             if service_signature_char:
@@ -799,10 +825,9 @@ class BlePairing(AbstractPairing):
                     # that presents identify as data.
                     decoded["format"] = "bool"
 
-                hap_char = s.add_char(normalized_uuid)
+                hap_char = s.add_char(normalized_uuid, iid=iid)
                 logger.debug("%s: char: %s decoded: %s", self.name, char, decoded)
-
-                hap_char.iid = iid
+                assert hap_char.iid == iid, "iid should be set"
                 hap_char.handle = char.handle
                 hap_char.perms = decoded["perms"]
                 # Some vendor characteristics have no format
@@ -1001,13 +1026,12 @@ class BlePairing(AbstractPairing):
         Older code did not save the handle, so if we have no handles
         we need to re-fetch the gatt database.
         """
-        try:
-            for service in self.accessories.aid(BLE_AID).services:
-                for char in service.characteristics:
-                    if char.handle is not None:
-                        return False
-        except StopIteration:
+        if not self.accessories.accessories:
             return True
+        for service in self.accessories.aid(BLE_AID).services:
+            for char in service.characteristics:
+                if char.handle is not None:
+                    return False
         return True
 
     async def _populate_accessories_and_characteristics(
@@ -1236,6 +1260,37 @@ class BlePairing(AbstractPairing):
     ) -> dict[tuple[int, int], dict[str, Any]]:
         return await self._get_characteristics_without_retry(characteristics)
 
+    def _sort_characteristics_by_fetch_order(
+        self, characteristics: list[Characteristic]
+    ) -> list[Characteristic]:
+        """Sort characteristics by fetch order.
+
+        Characteristics are sorted by what we expect to be the most
+        useful order to fetch them in. This is based on the following
+        assumptions:
+
+        The on/off state of the accessory is the most important
+        characteristic to fetch.
+
+        Color/Brightness/Color Temperature/etc are the next most important
+        characteristics to fetch.
+
+        The remaining characteristics are less important and can be
+        fetched in any order except for ones that are unlikely to change
+        frequently like the name, version, and thread status so they
+        are fetched last.
+        """
+        # If we ever fetch which service is the primary we should
+        # also sort by that.
+        return sorted(
+            characteristics,
+            key=lambda char: CHAR_FETCH_PRIORITY.get(
+                char.type,
+                10 if char.broadcast_events or char.disconnected_events else 0,
+            ),
+            reverse=True,
+        )
+
     @operation_lock
     @restore_connection_and_resume
     async def _get_characteristics_without_retry(
@@ -1250,15 +1305,19 @@ class BlePairing(AbstractPairing):
 
     async def _get_characteristics_while_connected(
         self,
-        characteristics: list[Characteristic],
+        unordered_characteristics: list[Characteristic],
         notify_listeners: bool = False,
     ) -> dict[tuple[int, int], dict[str, Any]]:
         assert self._operation_lock.locked(), "_operation_lock should be locked"
-        if logger.isEnabledFor(logging.DEBUG):
+        characteristics = self._sort_characteristics_by_fetch_order(
+            unordered_characteristics
+        )
+
+        debug_enabled = logger.isEnabledFor(logging.DEBUG)
+        if debug_enabled:
             logger.debug(
-                "%s: Reading characteristics: %s; rssi=%s",
+                "%s: Reading characteristics with rssi: %s",
                 self.name,
-                [char.iid for char in characteristics],
                 self.rssi,
             )
 
@@ -1274,7 +1333,14 @@ class BlePairing(AbstractPairing):
                     )
                     continue
 
-                logger.debug("%s: Reading characteristic %s", self.name, char.type)
+                if debug_enabled:
+                    logger.debug(
+                        "%s: Reading characteristic %s (%s) with iid %s",
+                        self.name,
+                        char.description,
+                        char.type,
+                        char.iid,
+                    )
 
                 try:
                     data = await self._async_request_under_lock(OpCode.CHAR_READ, char)
@@ -1284,15 +1350,17 @@ class BlePairing(AbstractPairing):
                     # we need to skip in this case.
                     if ex.status == PDUStatus.INVALID_REQUEST:
                         logger.debug(
-                            "%s: Reading characteristic %s with iid %s resulted in an invalid request (skipped)",
+                            "%s: Reading characteristic %s (%s) with iid %s resulted in an invalid request (skipped)",
                             self.name,
-                            char.iid,
+                            char.description,
                             char.type,
+                            char.iid,
                         )
                         continue
                     logger.exception(
-                        "%s: Reading characteristic %s with iid %s resulted in an error: %s",
+                        "%s: Reading characteristic %s (%s) with iid %s resulted in an error: %s",
                         self.name,
+                        char.description,
                         char.type,
                         char.iid,
                         ex,
@@ -1302,9 +1370,11 @@ class BlePairing(AbstractPairing):
                 decoded = dict(TLV.decode_bytes(data))[1]
 
                 logger.debug(
-                    "%s: Read characteristic %s got data, expected format is %s: data=%s decoded=%s",
+                    "%s: Read characteristic %s (%s) with iid %s got data, expected format is %s: data=%s decoded=%s",
                     self.name,
+                    char.description,
                     char.type,
+                    char.iid,
                     char.format,
                     data,
                     decoded,
@@ -1314,9 +1384,11 @@ class BlePairing(AbstractPairing):
                     value = from_bytes(char, decoded)
                 except struct.error as ex:
                     logger.debug(
-                        "%s: Failed to decode characteristic for %s from %s: %s",
+                        "%s: Failed to decode characteristic %s (%s) with iid %s from %s: %s",
                         self.name,
+                        char.description,
                         char.type,
+                        char.iid,
                         decoded,
                         ex,
                     )
@@ -1331,8 +1403,7 @@ class BlePairing(AbstractPairing):
                     # we want to notify the listeners as soon as we have the
                     # value for each characteristic.
                     single_results = {result_key: result_value}
-                    for listener in self.listeners:
-                        listener(single_results)
+                    self._callback_listeners(single_results)
 
         return results
 
@@ -1341,8 +1412,8 @@ class BlePairing(AbstractPairing):
     @disconnect_on_missing_services
     @restore_connection_and_resume
     async def put_characteristics(
-        self, characteristics: list[tuple[int, int, Any]]
-    ) -> dict[tuple[int, int], Any]:
+        self, characteristics: Iterable[tuple[int, int, Any]]
+    ) -> dict[tuple[int, int], dict[str, Any]]:
         results: dict[tuple[int, int], Any] = {}
         logger.debug(
             "%s: Writing characteristics: %s; rssi=%s",
@@ -1392,8 +1463,7 @@ class BlePairing(AbstractPairing):
                 # results only set on failure, no status is success
                 if not result:
                     if CharacteristicPermissions.paired_read in char.perms:
-                        for listener in self.listeners:
-                            listener({result_key: {"value": value}})
+                        self._callback_listeners({result_key: {"value": value}})
                 else:
                     results[result_key] = result
 
