@@ -374,3 +374,6 @@ class CharacteristicsTypes:
     VENDOR_AQARA_E1_GATEWAY_VOLUME = "EE56B186-B0D3-488E-8C79-C21FC9BCF437"
     # r/w, bool
     VENDOR_AQARA_E1_PAIRING_MODE = "B1C09E4C-E202-4827-B863-B0F32F727CFF"
+
+    # https://github.com/home-assistant/core/issues/73360 (r/o, dB)
+    VENDOR_NETATMO_NOISE = "B3BBFABC-D78C-5B8D-948C-5DAC1EE2CDE5"
